@@ -2,10 +2,13 @@
 
 import json
 import logging
+import time
 
 from splitio.api import APIException
-from splitio.api.commons import headers_from_metadata, build_fetch
+from splitio.api.commons import headers_from_metadata, build_fetch, record_telemetry
+from splitio.util.time import get_current_epoch_time_ms
 from splitio.api.client import HttpClientException
+from splitio.models.telemetry import HTTPExceptionsAndLatencies
 
 
 _LOGGER = logging.getLogger(__name__)
@@ -14,7 +17,7 @@ _LOGGER = logging.getLogger(__name__)
 class SegmentsAPI(object):  # pylint: disable=too-few-public-methods
     """Class that uses an httpClient to communicate with the segments API."""
 
-    def __init__(self, http_client, apikey, sdk_metadata):
+    def __init__(self, http_client, apikey, sdk_metadata, telemetry_runtime_producer):
         """
         Class constructor.
 
@@ -29,6 +32,7 @@ class SegmentsAPI(object):  # pylint: disable=too-few-public-methods
         self._client = http_client
         self._apikey = apikey
         self._metadata = headers_from_metadata(sdk_metadata)
+        self._telemetry_runtime_producer = telemetry_runtime_producer
 
     def fetch_segment(self, segment_name, change_number, fetch_options):
         """
@@ -46,6 +50,7 @@ class SegmentsAPI(object):  # pylint: disable=too-few-public-methods
         :return: Json representation of a segmentChange response.
         :rtype: dict
         """
+        start = get_current_epoch_time_ms()
         try:
             query, extra_headers = build_fetch(change_number, fetch_options, self._metadata)
             response = self._client.get(
@@ -55,7 +60,7 @@ class SegmentsAPI(object):  # pylint: disable=too-few-public-methods
                 extra_headers=extra_headers,
                 query=query,
             )
-
+            record_telemetry(response.status_code, get_current_epoch_time_ms() - start, HTTPExceptionsAndLatencies.SEGMENT, self._telemetry_runtime_producer)
             if 200 <= response.status_code < 300:
                 return json.loads(response.body)
             else:
