@@ -7,7 +7,7 @@ import time
 
 from splitio.api import APIException
 from splitio.util.backoff import Backoff
-
+from splitio.sync.split import _ON_DEMAND_FETCH_BACKOFF_BASE, _ON_DEMAND_FETCH_BACKOFF_MAX_RETRIES, _ON_DEMAND_FETCH_BACKOFF_MAX_WAIT, LocalhostMode
 
 _LOGGER = logging.getLogger(__name__)
 _SYNC_ALL_NO_RETRIES = -1
@@ -16,7 +16,7 @@ class SplitSynchronizers(object):
     """SplitSynchronizers."""
 
     def __init__(self, split_sync, segment_sync, impressions_sync, events_sync,  # pylint:disable=too-many-arguments
-                 impressions_count_sync, unique_keys_sync = None, clear_filter_sync = None):
+                 impressions_count_sync, telemetry_sync=None, unique_keys_sync = None, clear_filter_sync = None):
         """
         Class constructor.
 
@@ -38,6 +38,7 @@ class SplitSynchronizers(object):
         self._impressions_count_sync = impressions_count_sync
         self._unique_keys_sync = unique_keys_sync
         self._clear_filter_sync = clear_filter_sync
+        self._telemetry_sync = telemetry_sync
 
     @property
     def split_sync(self):
@@ -74,11 +75,16 @@ class SplitSynchronizers(object):
         """Return clear filter synchonizer."""
         return self._clear_filter_sync
 
+    @property
+    def telemetry_sync(self):
+        """Return clear filter synchonizer."""
+        return self._telemetry_sync
+
 class SplitTasks(object):
     """SplitTasks."""
 
     def __init__(self, split_task, segment_task, impressions_task, events_task,  # pylint:disable=too-many-arguments
-                 impressions_count_task, unique_keys_task = None, clear_filter_task = None):
+                 impressions_count_task, telemetry_task=None, unique_keys_task = None, clear_filter_task = None):
         """
         Class constructor.
 
@@ -100,6 +106,7 @@ class SplitTasks(object):
         self._impressions_count_task = impressions_count_task
         self._unique_keys_task = unique_keys_task
         self._clear_filter_task = clear_filter_task
+        self._telemetry_task = telemetry_task
 
     @property
     def split_task(self):
@@ -136,6 +143,11 @@ class SplitTasks(object):
         """Return clear filter sync task."""
         return self._clear_filter_task
 
+    @property
+    def telemetry_task(self):
+        """Return clear filter sync task."""
+        return self._telemetry_task
+
 class BaseSynchronizer(object, metaclass=abc.ABCMeta):
     """Synchronizer interface."""
 
@@ -214,9 +226,6 @@ class BaseSynchronizer(object, metaclass=abc.ABCMeta):
 class Synchronizer(BaseSynchronizer):
     """Synchronizer."""
 
-    _ON_DEMAND_FETCH_BACKOFF_BASE = 10  # backoff base starting at 10 seconds
-    _ON_DEMAND_FETCH_BACKOFF_MAX_WAIT = 30  # don't sleep for more than 1 minute
-
     def __init__(self, split_synchronizers, split_tasks):
         """
         Class constructor.
@@ -227,13 +236,14 @@ class Synchronizer(BaseSynchronizer):
         :type split_tasks: splitio.sync.synchronizer.SplitTasks
         """
         self._backoff = Backoff(
-                                self._ON_DEMAND_FETCH_BACKOFF_BASE,
-                                self._ON_DEMAND_FETCH_BACKOFF_MAX_WAIT)
+                                _ON_DEMAND_FETCH_BACKOFF_BASE,
+                                _ON_DEMAND_FETCH_BACKOFF_MAX_WAIT)
         self._split_synchronizers = split_synchronizers
         self._split_tasks = split_tasks
         self._periodic_data_recording_tasks = [
             self._split_tasks.impressions_task,
-            self._split_tasks.events_task
+            self._split_tasks.events_task,
+            self._split_tasks.telemetry_task
         ]
         if self._split_tasks.impressions_count_task:
             self._periodic_data_recording_tasks.append(self._split_tasks.impressions_count_task)
@@ -242,7 +252,6 @@ class Synchronizer(BaseSynchronizer):
         if self._split_tasks.clear_filter_task:
             self._periodic_data_recording_tasks.append(self._split_tasks.clear_filter_task)
 
-
     def _synchronize_segments(self):
         _LOGGER.debug('Starting segments synchronization')
         return self._split_synchronizers.segment_sync.synchronize_segments()
@@ -306,6 +315,7 @@ class Synchronizer(BaseSynchronizer):
                     raise Exception("split sync failed")
 
                 # Only retrying splits, since segments may trigger too many calls.
+
                 if not self._synchronize_segments():
                     _LOGGER.warning('Segments failed to synchronize.')
 
@@ -367,10 +377,14 @@ class Synchronizer(BaseSynchronizer):
         if blocking:
             events = []
             for task in self._periodic_data_recording_tasks:
-                stop_event = threading.Event()
-                task.stop(stop_event)
-                events.append(stop_event)
-            if all(event.wait() for event in events):
+                if task != self._split_tasks.telemetry_task:
+                    stop_event = threading.Event()
+                    task.stop(stop_event)
+                    events.append(stop_event)
+            all(event.wait() for event in events)
+            telemetry_event = threading.Event()
+            self._split_tasks.telemetry_task.stop(telemetry_event)
+            if telemetry_event.wait():
                 _LOGGER.debug('all tasks finished successfully.')
         else:
             for task in self._periodic_data_recording_tasks:
@@ -410,6 +424,7 @@ class RedisSynchronizer(BaseSynchronizer):
             self._tasks.append(split_tasks.unique_keys_task)
         if split_tasks.clear_filter_task is not None:
             self._tasks.append(split_tasks.clear_filter_task)
+        self._periodic_data_recording_tasks = []
 
     def sync_all(self):
         """
@@ -476,7 +491,7 @@ class RedisSynchronizer(BaseSynchronizer):
 class LocalhostSynchronizer(BaseSynchronizer):
     """LocalhostSynchronizer."""
 
-    def __init__(self, split_synchronizers, split_tasks):
+    def __init__(self, split_synchronizers, split_tasks, localhost_mode):
         """
         Class constructor.
 
@@ -487,40 +502,76 @@ class LocalhostSynchronizer(BaseSynchronizer):
         """
         self._split_synchronizers = split_synchronizers
         self._split_tasks = split_tasks
+        self._localhost_mode = localhost_mode
+        self._backoff = Backoff(
+                                _ON_DEMAND_FETCH_BACKOFF_BASE,
+                                _ON_DEMAND_FETCH_BACKOFF_MAX_WAIT)
 
-    def sync_all(self, max_retry_attempts=-1):
+    def sync_all(self, till=None):
         """
         Synchronize all splits.
-
-        :param max_retry_attempts: Not used, added for compatibility
         """
-        try:
-            self._split_synchronizers.split_sync.synchronize_splits(None)
-        except APIException as exc:
-            _LOGGER.error('Failed syncing splits')
-            raise APIException('Failed to sync splits') from exc
+        # TODO: to be removed when legacy and yaml use BUR
+        if self._localhost_mode != LocalhostMode.JSON:
+            return self.synchronize_splits()
+
+        self._backoff.reset()
+        remaining_attempts = _ON_DEMAND_FETCH_BACKOFF_MAX_RETRIES
+        while remaining_attempts > 0:
+            remaining_attempts -= 1
+            try:
+                return self.synchronize_splits()
+            except APIException as exc:
+                _LOGGER.error('Failed syncing all')
+                _LOGGER.error(str(exc))
+
+            how_long = self._backoff.get()
+            time.sleep(how_long)
 
     def start_periodic_fetching(self):
         """Start fetchers for splits and segments."""
-        _LOGGER.debug('Starting periodic data fetching')
-        self._split_tasks.split_task.start()
+        if self._split_tasks.split_task is not None:
+            _LOGGER.debug('Starting periodic data fetching')
+            self._split_tasks.split_task.start()
+        if self._split_tasks.segment_task is not None:
+            self._split_tasks.segment_task.start()
 
     def stop_periodic_fetching(self):
         """Stop fetchers for splits and segments."""
-        _LOGGER.debug('Stopping periodic fetching')
-        self._split_tasks.split_task.stop()
+        if self._split_tasks.split_task is not None:
+            _LOGGER.debug('Stopping periodic fetching')
+            self._split_tasks.split_task.stop()
+        if self._split_tasks.segment_task is not None:
+            self._split_tasks.segment_task.stop()
 
     def kill_split(self, split_name, default_treatment, change_number):
         """Kill a split locally."""
         raise NotImplementedError()
 
-    def synchronize_splits(self, till):
+    def synchronize_splits(self):
         """Synchronize all splits."""
-        raise NotImplementedError()
+        try:
+            new_segments = []
+            for segment in self._split_synchronizers.split_sync.synchronize_splits():
+                    if not self._split_synchronizers.segment_sync.segment_exist_in_storage(segment):
+                        new_segments.append(segment)
+            if len(new_segments) > 0:
+                _LOGGER.debug('Synching Segments: %s', ','.join(new_segments))
+                success = self._split_synchronizers.segment_sync.synchronize_segments(new_segments)
+                if not success:
+                    _LOGGER.error('Failed to schedule sync one or all segment(s) below.')
+                    _LOGGER.error(','.join(new_segments))
+                else:
+                    _LOGGER.debug('Segment sync scheduled.')
+            return True
+
+        except APIException as exc:
+            _LOGGER.error('Failed syncing splits')
+            raise APIException('Failed to sync splits') from exc
 
     def synchronize_segment(self, segment_name, till):
         """Synchronize particular segment."""
-        raise NotImplementedError()
+        pass
 
     def start_periodic_data_recording(self):
         """Start recorders."""
